<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xlink="http://www.w3.org/1999/xlink">

  <xsl:variable name="custom-css">
    <xsl:for-each select="/div/@type">
      <link  href="{.}.css" type="text/css" rel="stylesheet" />
    </xsl:for-each>
  </xsl:variable>
  
  <xsl:template match="/">
    <html>
      <head>
        <title>
          <xsl:value-of select="/descendant::*[@type='title'][1]"/>
        </title>
        <style type="text/css">

p.line { margin: 0px }
div.div { margin-top: 1ex; margin-bottom: 1ex }

table.ap td, table.ap th { vertical-align: top }

        </style>
        <xsl:copy-of select="$custom-css"/>
      </head>
      <body>
        <xsl:apply-templates/>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="div | span">
    <xsl:copy>
      <xsl:call-template name="attributes"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="line[@type='head']">
    <xsl:variable name="level" select="count(ancestor::*[*[1]/@type='head'])"/>
    <xsl:element name="h{$level}">
      <xsl:call-template name="attributes"/>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="ab">
    <div>
      <xsl:call-template name="attributes"/>
      <xsl:apply-templates/>
    </div>
  </xsl:template>

  <xsl:template match="ap">
    <table>
      <xsl:call-template name="attributes"/>
      <xsl:apply-templates/>
    </table>
  </xsl:template>
  
  <xsl:template match="ap/pair">
    <tr>
      <xsl:call-template name="attributes"/>
      <xsl:apply-templates/>
    </tr>
  </xsl:template>

  <xsl:template match="ap/pair/*">
    <td>
      <xsl:call-template name="attributes"/>
      <xsl:apply-templates/>
    </td>
  </xsl:template>

  <xsl:template match="pair">
    <span>
      <xsl:call-template name="attributes"/>
      <xsl:apply-templates/>
    </span>
  </xsl:template>


  <xsl:template match="link">
    <a>
      <xsl:call-template name="attributes"/>
      <xsl:apply-templates/>
    </a>
  </xsl:template>
  
  <xsl:template match="line">
    <p>
      <xsl:call-template name="attributes"/>
      <xsl:apply-templates/>
    </p>
  </xsl:template>

  <xsl:template name="attributes">
    <xsl:apply-templates select="@*"/>
    <xsl:variable name="assigned-class">
      <xsl:value-of select="name()"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="@class"/>
      <xsl:text> </xsl:text>
      <xsl:for-each select="@type">
        <xsl:value-of select="."/>
        <xsl:text>-</xsl:text>
        <xsl:value-of select="name(..)"/>
      </xsl:for-each>
    </xsl:variable>
    <xsl:attribute name="class">
      <xsl:value-of select="normalize-space($assigned-class)"/>
    </xsl:attribute>
  </xsl:template>

  <xsl:template match="@style | @xml:id">
    <xsl:copy-of select="."/>
  </xsl:template>
  
  <xsl:template match="@href">
    <xsl:attribute name="href">
      <xsl:value-of select="."/>
    </xsl:attribute>
  </xsl:template>
  
  <xsl:template match="@*"/>
    
</xsl:stylesheet>
